User's Guide
============

.. toctree::
   :maxdepth: 2

   getting_started
   usage
   configuration
   differences
   plugins
   tools
   changelog

Plugin Developer's Guide
========================

.. toctree ::
   :maxdepth: 2

   dev/writing_plugins
   dev/documenting_plugins
   dev/event_reference
   dev/hook_reference
   dev/session_reference
   dev/plugin_class_reference

Developer's Guide
=================

.. toctree::
   :maxdepth: 2

   dev/contributing
   dev/internals

Indices and tables
==================

* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`
